/***************************************************
Это пример программы для цветного OLED экрана основанного на драйвере SSD1331 
 ****************************************************/

#include <Adafruit_GFX.h>
#include <Adafruit_SSD1331.h>

// Номера подключенных пинов
#define sclk 13
#define mosi 11
#define cs   10
#define rst  9
#define dc   8

//Определяем Цвета
#define	BLACK           0x0000
#define	BLUE            0x001F
#define	RED             0xF800
#define	GREEN           0x07E0
#define CYAN            0x07FF
#define MAGENTA         0xF81F
#define YELLOW          0xFFE0
#define WHITE           0xFFFF

//  Создаем объект дисплея SSD1331
Adafruit_SSD1331 display = Adafruit_SSD1331(cs, dc, rst);

int example_delay = 2000; //задержка в миллисекундах между примерами
int step_delay = 100; //задержка в миллисекундах между шагами одного примера

#define LOGONUM     10 // Количество картинок в анимации

#define LOGO_HEIGHT   20
#define LOGO_WIDTH    20
static const unsigned char PROGMEM logo_bmp[] =
{ 
  0x00,0x00,0x0F, // ....................####
  0x00,0x00,0x0F, // ....................####
  0x3E,0x08,0x0F, // ..#####.....#.......####
  0x41,0x90,0x0F, // .#.....##..#........####
  0x60,0x5E,0x0F, // .##......#.####.....####
  0x18,0x71,0x0F, // ...##....###...#....####
  0x0F,0x80,0x8F, // ....#####.......#...####
  0x08,0x00,0x4F, // ....#............#..####
  0x10,0x00,0x4F, // ...#.............#..####
  0x10,0x00,0x4F, // ...#.............#..####
  0x10,0x00,0x4F, // ...#.............#..####
  0x10,0x00,0x4F, // ...#.............#..####
  0x10,0x00,0x4F, // ...#.............#..####
  0x08,0x00,0x4F, // ....#............#..####
  0x08,0x00,0x8F, // ....#...........#...####
  0x04,0x01,0x0F, // .....#.........#....####
  0x02,0x02,0x0F, // ......#.......#.....####
  0x01,0xFC,0x0F, // .......#######......####
  0x00,0x00,0x0F, // ....................####
  0x00,0x00,0x0F  // ....................####
};

void setup(void) {
  Serial.begin(9600);
  display.begin(); // Инициализирует дисплей
  Serial.println("init");
  
  example_fill_screen(); //пример заполнения экрана цветом

  example_show_text(); //пример вывода текста на экран
  
  example_draw_pixel(); //пример рисования точек

  example_draw_line(); //пример рисования линий

  example_draw_rect(); //пример рисования прямоугольников

  example_draw_circle(); // пример рисования кругов

  example_draw_triangle(); // пример рисования треугольников
  
  example_draw_bitmap();    // выводим картинку

  example_animate_bitmap(logo_bmp, LOGO_WIDTH, LOGO_HEIGHT); //анимируем картинку
  
  the_end();
}

void loop() {
}

String utf8rus(String source)
{
   int i,k;
   String target;
   unsigned char n;
   char m[2] = { '0', '\0' };

   k = source.length(); i = 0;
   while (i < k) 
   {
      n = source[i]; i++;
      if (n >= 0xC0) 
      {
         switch (n) 
         {
            case 0xD0: 
            {
               n = source[i]; i++;
               if (n == 0x81) { n = 0xA8; break; }
               if (n >= 0x90 && n <= 0xBF) n = n + 0x2F;
               break;
            }
           case 0xD1: 
            {
                n = source[i]; i++;
                if (n == 0x91) { n = 0xB8; break; }
                if (n >= 0x80 && n <= 0x8F) n = n + 0x6F;
                break;
            }
         }
      }
      m[0] = n; target = target + String(m);
   }
   return target;
}

void example_fill_screen(void) {
  
  display.fillScreen(BLACK);
  /*выводим пояснительный текст*/  
  display.setTextSize(1);            
  display.setTextColor(RED);      
  display.setCursor(0,0);          
  display.println(utf8rus("Заливка экрана"));
  delay(example_delay/2);
  
   display.fillScreen(RED);
  /*выводим пояснительный текст*/  
  display.setTextColor(GREEN);
  display.setCursor(0,0);      
  display.println(utf8rus("Заливка экрана"));
  delay(example_delay/2);

  display.fillScreen(GREEN);
  /*выводим пояснительный текст*/  
  display.setTextColor(CYAN);  
  display.setCursor(0,0);    
  display.println(utf8rus("Заливка экрана"));
  delay(example_delay/2);

  display.fillScreen(CYAN);
  /*выводим пояснительный текст*/  
  display.setTextColor(MAGENTA);  
  display.setCursor(0,0);    
  display.println(utf8rus("Заливка экрана"));
  delay(example_delay/2);

  display.fillScreen(MAGENTA);
  /*выводим пояснительный текст*/  
  display.setTextColor(YELLOW); 
  display.setCursor(0,0);     
  display.println(utf8rus("Заливка экрана"));
  delay(example_delay/2);

  display.fillScreen(YELLOW);
  /*выводим пояснительный текст*/  
  display.setTextColor(WHITE);  
  display.setCursor(0,0);    
  display.println(utf8rus("Заливка экрана"));
  delay(example_delay/2);
  
  display.fillScreen(WHITE);
  /*выводим пояснительный текст*/  
  display.setTextColor(BLUE); 
  display.setCursor(0,0);     
  display.println(utf8rus("Заливка экрана"));
  delay(example_delay/2);

  display.fillScreen(BLUE);
  /*выводим пояснительный текст*/  
  display.setTextColor(BLACK); 
  display.setCursor(0,0);     
  display.println(utf8rus("Заливка экрана"));
  delay(example_delay);
}

void example_show_text(void) {
  display.fillScreen(BLACK);
  display.setCursor(0, 0);
  display.setTextColor(RED);
  display.setTextSize(1);
  display.println(utf8rus("Мы напишем"));
  delay(step_delay);
  display.setCursor(0, 15);
  display.setTextColor(YELLOW, GREEN);
  display.setTextSize(2);
  display.print(utf8rus("Классный текст"));
  delay(step_delay);
  display.setCursor(0, 50);
  display.setTextColor(BLUE,WHITE);
  display.setTextSize(1);
  display.print(utf8rus("Для примера"));
  delay(example_delay);  
}

 void example_draw_pixel(void) {
 
  /*выводим пояснительный текст*/  
  display.fillScreen(BLACK);
  display.setTextSize(1);            
  display.setTextColor(RED);      
  display.setCursor(0,0);          
  display.println(utf8rus("Рисуем точки"));
  delay(example_delay/2);

  /*выводим точки*/
  display.fillScreen(BLACK); 
  int16_t i, j = display.height()/2; 

  for(i=0; i<display.width(); i+=10) {
    if(i<display.width()/3)
       display.drawPixel(i, j, RED);
    else if( (i>=display.width()/3)&&(i<display.width()/3*2) )
       display.drawPixel(i, j, GREEN);
    else if(i>=display.width()/3*2)
       display.drawPixel(i, j, BLUE);
   
    delay(step_delay);
  }

  delay(example_delay);
}

void example_draw_line(void) {

  /*выводим пояснительный текст*/
  display.fillScreen(BLACK);
  display.setTextSize(1);            
  display.setTextColor(GREEN);      
  display.setCursor(0,0);          
  display.println(utf8rus("Рисуем линии"));
  delay(example_delay/2);

  int16_t i;

  /*выводим вертикальные линии*/

  display.fillScreen(BLACK);

   for (int16_t y=0; y < display.height()-1; y+=4) {
     display.drawFastHLine(0, y, display.width()-1, CYAN);
     delay(step_delay);
   }
   for (int16_t x=0; x < display.width()-1; x+=8) {
     display.drawFastVLine(x, 0, display.height()-1, YELLOW);
     delay(step_delay);
   }

  delay(example_delay/2);

  /*выводим диагональные линии*/

  display.fillScreen(BLACK);
 
  for(i=0; i<display.height(); i+=4) {
    display.drawLine(display.width()-1, 0, 0, i, MAGENTA);
    delay(step_delay);
  }
   for(i=8; i<display.width(); i+=8) {
    display.drawLine(display.width()-1, 0, i, display.height()-1, RED);
   delay(step_delay);
  }
 
  delay(example_delay/2);

  display.fillScreen(BLACK);

   for(i=0; i<display.width()-1; i+=8) {
    display.drawLine(i, 0, display.width()-1-i, display.height()-1, BLUE);
    delay(step_delay);
  }
   for(i=4; i<display.height()-1; i+=4) {
    display.drawLine(display.width()-1, i, 0, display.height()-1-i, CYAN);
    delay(step_delay);
  } 
  
  delay(example_delay); 
}

void example_draw_rect(void) {
  
  /*выводим пояснительный текст*/
  display.fillScreen(BLACK);
  display.setTextSize(1);            
  display.setTextColor(BLUE);      
  display.setCursor(0,0);          
  display.println(utf8rus("Рисуем прямоугольники"));
  delay(example_delay/2);

  /*выводим прямогугольники вложенные друг в друга*/
  
 display.fillScreen(BLACK);

 for(int16_t i=0; i<display.height()/2; i+=2) {
    display.drawRect(i, i, display.width()-2*i, display.height()-2*i, BLUE);
    delay(step_delay);
  }

  /*выводим заполненные цветом прямоугольники*/  
  display.fillScreen(BLACK);
  for(int16_t i=0; i<display.width()-1; i+=16) {
    display.fillRect(i, 0, 15, display.height()-1, MAGENTA);
    display.drawRect(i, 0, 15, display.height()-1, BLUE);
    delay(step_delay);
  }

  delay(example_delay/2);

   /*выводим скругленные прямоугольники*/
   display.fillScreen(BLACK);
 
   for(int16_t i=0; i<display.height()/2-2; i+=2) {
    display.drawRoundRect(i, i, display.width()-2*i, display.height()-2*i,
    display.height()/4, RED);
    delay(step_delay);
  }

  delay(example_delay/2);

  /*выводим заполненные цветом прямоугольники*/  
   display.fillScreen(BLACK);
   
  for(int16_t i=0; i<display.width()-1; i+=16) {
    display.fillRoundRect(i, 0, 15, display.height()-1, display.height()/4, GREEN);
    display.drawRoundRect(i, 0, 15, display.height()-1, display.height()/4, RED);
    delay(step_delay);
  }

  delay(example_delay);
}

void example_draw_circle(void) {

  /*выводим пояснительный текст*/
  display.fillScreen(BLACK);
  display.setTextSize(1);            
  display.setTextColor(CYAN);      
  display.setCursor(0,0);          
  display.println(utf8rus("Рисуем круги"));
  delay(example_delay/2);

  /*выводим круги*/
  display.fillScreen(BLACK);

  int16_t d = display.height()/2-1;
  
   for(int16_t i=d/2; i<=display.width()-d/5; i+=d) {
    display.drawCircle(i, d/2, d/2, MAGENTA);
    delay(step_delay);
  }
  
  for(int16_t i=d; i<=display.width()-d; i+=d) {
    display.drawCircle(i, d+d/2, d/2, MAGENTA);
    delay(step_delay);
  }

  delay(example_delay/2);

  /*выводим заполненные круги*/
  display.fillScreen(BLACK);

  for(int16_t i=d/2; i<=display.width()-d/2; i+=d) {
    display.fillCircle(i, d/2, d/2, CYAN);
    display.drawCircle(i, d/2, d/2, MAGENTA);
    delay(step_delay);
  }
  
  for(int16_t i=d; i<=display.width()-d; i+=d) {
    display.fillCircle(i, d+d/2, d/2, CYAN);
    display.drawCircle(i, d+d/2, d/2, MAGENTA);
    delay(step_delay);
  }
  
  delay(example_delay);
}

void example_draw_triangle(void) {

  /*выводим пояснительный текст*/
  display.fillScreen(BLACK);
     
  display.setTextSize(1);            
  display.setTextColor(YELLOW);      
  display.setCursor(0,0);          
  display.println(utf8rus("Рисуем треугольники"));
  delay(example_delay/2);

  /*выводим треугольники*/
   display.fillScreen(BLACK);

  int16_t d = display.height()/2-1;
  
  for(int16_t i=0; i<display.width()-d/2; i+=d) {
    display.drawTriangle(
    i, 0,
    i+d/2,d,
    i+d, 0, GREEN);
    delay(step_delay);
  }

  for(int16_t i=d/2; i<display.width()-d; i+=d) {
    display.drawTriangle(
    i, display.height()-1,
    i+d/2,d,
    i+d, display.height()-1, GREEN);
    delay(step_delay);
  }

  delay(example_delay/2);

  /*выводим треугольники заполненные цветом*/
   display.fillScreen(BLACK);

 for(int16_t i=0; i<display.width()-d/2; i+=d) {
    display.fillTriangle(
    i, 0,
    i+d/2,d,
    i+d, 0, YELLOW);
    delay(step_delay);
  }

  for(int16_t i=d/2; i<display.width()-d; i+=d) {
    display.fillTriangle(
    i, display.height()-1,
    i+d/2,d,
    i+d, display.height()-1, YELLOW);
    delay(step_delay);
  }  

  delay(example_delay);
}

void example_draw_bitmap(void) {
  /*выводим пояснительный текст*/
  display.fillScreen(BLACK);
  display.setTextSize(1);            
  display.setTextColor(MAGENTA);      
  display.setCursor(0,0);          
  display.println(utf8rus("Рисуем картинку"));
  delay(example_delay/2);

  /*выводим картинку*/
  
  display.fillScreen(BLACK);
  display.drawBitmap(
    (display.width()  - LOGO_WIDTH ) / 2,
    (display.height() - LOGO_HEIGHT) / 2,
    logo_bmp, LOGO_WIDTH, LOGO_HEIGHT, RED);
  delay(example_delay*2);
}

#define XPOS   0 // Индексирует в массив 'icons' в функции ниже
#define YPOS   1
#define DELTAY 2

void example_animate_bitmap(const uint8_t *bitmap, uint8_t w, uint8_t h) {

  /*выводим пояснительный текст*/
  display.fillScreen(BLACK);
 
  display.setTextSize(1);            
  display.setTextColor(WHITE);      
  display.setCursor(0,0);          
  display.println(utf8rus("Анимация картинки"));
  delay(example_delay/2);

  /*выводим анимацию*/
  display.fillScreen(BLACK);
  
  int8_t i, f, icons[LOGONUM][3];

  // Инициируем позиции картинок
  for(f=0; f< LOGONUM; f++) {
    icons[f][XPOS]   = random(1 - LOGO_WIDTH, display.width());
    icons[f][YPOS]   = -LOGO_HEIGHT;
    icons[f][DELTAY] = random(1, 6);
  }

  for(i=0;i<50;i++) { //цикл анимации 50 итераций
     display.fillScreen(BLACK);

    // Рисуем каждую картинку:
    for(f=0; f< LOGONUM; f++) {
      display.drawBitmap(icons[f][XPOS], icons[f][YPOS], bitmap, w, h, RED);
    }

    delay(step_delay);        // пауза

    // обновляем координаты каждой картинки...
    for(f=0; f< LOGONUM; f++) {
      icons[f][YPOS] += icons[f][DELTAY];
      // если картинка находится внизу экрана...
      if (icons[f][YPOS] >= display.height()) {
        // Снова инициализируем в случайную позицию
        icons[f][XPOS]   = random(1 - LOGO_WIDTH, display.width());
        icons[f][YPOS]   = -LOGO_HEIGHT;
        icons[f][DELTAY] = random(1, 6);
      }
    }
  }
}

void the_end(void) {
  display.fillScreen(BLACK);
  /*выводим пояснительный текст*/

  display.setTextSize(1);            
  display.setTextColor(BLUE);      
  display.setCursor(0,0);          
  display.println(utf8rus("Спасибо за внимание!"));
}
